@@ -7,7 +7,6 @@ package main
 import (
 	"bytes"
 	"encoding/xml"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -68,7 +67,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	}
 	iqDiscoItemsXMLQuery = iqDiscoItemsXML.SelectElement("query")
 	if iqDiscoItemsXMLQuery == nil {
-		return "", errors.New("http-upload: no query element in disco items reply")
+		return "", fmt.Errorf("http-upload: no query element in disco items reply")
 	}
 	iqDiscoItemsXMLItems := iqDiscoItemsXMLQuery.SelectElements("item")
 
@@ -118,7 +117,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 		}
 	}
 	if uploadComponent == "" {
-		return "", errors.New("http-upload: no http upload component found.")
+		return "", fmt.Errorf("http-upload: no http upload component found.")
 	}
 	iqDiscoInfoXMLX := iqDiscoInfoXMLQuery.SelectElements("x")
 	for _, r := range iqDiscoInfoXMLX {
@@ -143,7 +142,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 				if prevFieldVal.Text() == nsHTTPUpload {
 					maxFileSize, err = strconv.ParseInt(curFieldVal.Text(), 10, 64)
 					if err != nil {
-						return "", errors.New("http-upload: error while checking server maximum http upload file size.")
+						return "", fmt.Errorf("http-upload: error while checking server maximum http upload file size.")
 					}
 				}
 			}
@@ -154,9 +153,8 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	// the best.
 	if maxFileSize != 0 {
 		if fileSize > maxFileSize {
-			return "", errors.New("http-upload: file size " + strconv.FormatInt(fileSize/1024/1024, 10) +
-				" MiB is larger than the maximum file size allowed (" +
-				strconv.FormatInt(maxFileSize/1024/1024, 10) + " MiB).")
+			return "", fmt.Errorf("http-upload: file size %s MiB is larger than the maximum file size allowed (%s MiB).",
+				strconv.FormatInt(fileSize/1024/1024, 10), strconv.FormatInt(maxFileSize/1024/1024, 10))
 		}
 	}
 
@@ -178,7 +176,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 		return "", err
 	}
 	if uploadSlot.Type != strResult {
-		return "", errors.New("http-upload: error while requesting upload slot.")
+		return "", fmt.Errorf("http-upload: error while requesting upload slot.")
 	}
 	iqHTTPUploadSlotXML := etree.NewDocument()
 	err = iqHTTPUploadSlotXML.ReadFromBytes(uploadSlot.Query)
@@ -187,18 +185,18 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	}
 	iqHTTPUploadSlotXMLSlot := iqHTTPUploadSlotXML.SelectElement("slot")
 	if iqHTTPUploadSlotXMLSlot == nil {
-		return "", errors.New("http-upload: no slot element")
+		return "", fmt.Errorf("http-upload: no slot element")
 	}
 	iqHTTPUploadSlotXMLPut := iqHTTPUploadSlotXMLSlot.SelectElement("put")
 	if iqHTTPUploadSlotXMLPut == nil {
-		return "", errors.New("http-upload: no put element")
+		return "", fmt.Errorf("http-upload: no put element")
 	}
 	iqHTTPUploadSlotXMLPutURL := iqHTTPUploadSlotXMLPut.SelectAttr("url")
 	if iqHTTPUploadSlotXMLPutURL == nil {
-		return "", errors.New("http-upload: no url attribute")
+		return "", fmt.Errorf("http-upload: no url attribute")
 	}
 	if !strings.HasPrefix(iqHTTPUploadSlotXMLPutURL.Value, "https://") {
-		return "", errors.New("http-upload: upload slot does not provide https")
+		return "", fmt.Errorf("http-upload: upload slot does not provide https")
 	}
 	// Upload file
 	httpTransport := &http.Transport{
@@ -237,17 +235,17 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	}
 	// Test for http status code "200 OK" or "201 Created"
 	if resp.StatusCode != 200 && resp.StatusCode != 201 {
-		return "", errors.New("http-upload: upload failed.")
+		return "", fmt.Errorf("http-upload: upload failed.")
 	}
 
 	// Return http link
 	iqHTTPUploadSlotXMLGet := iqHTTPUploadSlotXMLSlot.SelectElement("get")
 	if iqHTTPUploadSlotXMLGet == nil {
-		return "", errors.New("http-upload: no get element")
+		return "", fmt.Errorf("http-upload: no get element")
 	}
 	iqHTTPUploadSlotXMLGetURL := iqHTTPUploadSlotXMLGet.SelectAttr("url")
 	if iqHTTPUploadSlotXMLGetURL == nil {
-		return "", errors.New("http-upload: no url attribute")
+		return "", fmt.Errorf("http-upload: no url attribute")
 	}
 	err = resp.Body.Close()
 	if err != nil {
