@@ -8,7 +8,7 @@
 package main
 
 import (
-	"errors"
+	"fmt"
 	"strings"
 	"unicode/utf8"
 )
@@ -35,7 +35,7 @@ func MarshalJID(input string) (string, error) {
 	} else {
 		// If the resource part exists, make sure it isn't empty.
 		if sep == len(s)-1 {
-			return input, errors.New("Invalid JID" + input + ": The resourcepart must be larger than 0 bytes")
+			return input, fmt.Errorf("invalid JID %s: the resourcepart must be larger than 0 bytes", input)
 		}
 		resourcepart = s[sep+1:]
 		s = s[:sep]
@@ -52,7 +52,7 @@ func MarshalJID(input string) (string, error) {
 		domainpart = s
 	case sep == 0:
 		// The JID starts with an @ sign (invalid empty localpart)
-		err = errors.New("Invalid JID:" + input)
+		err = fmt.Errorf("Invalid JID: %s", input)
 		return input, err
 	default:
 		domainpart = s[sep+1:]
@@ -73,11 +73,11 @@ func MarshalJID(input string) (string, error) {
 	var jid string
 
 	if !utf8.ValidString(localpart) || !utf8.ValidString(domainpart) || !utf8.ValidString(resourcepart) {
-		return input, errors.New("Invalid JID: " + input)
+		return input, fmt.Errorf("invalid JID: %s", input)
 	}
 
 	if domainpart == "" {
-		return input, errors.New("Invalid JID: " + input)
+		return input, fmt.Errorf("invalid JID: %s", input)
 	}
 
 	if localpart == "" {
