@@ -8,7 +8,6 @@ import (
 	"bufio"
 	"context"
 	"crypto/tls"
-	"errors"
 	"fmt"
 	"io"
 	"log"
@@ -58,6 +57,7 @@ func readMessage(messageFilePath string) (string, error) {
 	if err != nil {
 		return output, fmt.Errorf("readMessage: %w", err)
 	}
+	defer file.Close()
 	scanner := bufio.NewScanner(file)
 	scanner.Split(bufio.ScanLines)
 	for scanner.Scan() {
@@ -74,11 +74,6 @@ func readMessage(messageFilePath string) (string, error) {
 		}
 	}
 
-	err = file.Close()
-	if err != nil {
-		fmt.Println("error while closing file:", err)
-	}
-
 	return output, nil
 }
 
@@ -136,6 +131,8 @@ func main() {
 	flagHeadline := getopt.BoolLong("headline", 0, "Send message as type headline.")
 	flagSCRAMPinning := getopt.StringLong("scram-mech-pinning", 0, "", "Enforce the use of a certain SCRAM authentication mechanism.")
 	flagSSDPOff := getopt.BoolLong("ssdp-off", 0, "Disable XEP-0474: SASL SCRAM Downgrade Protection.")
+	flagSubject := getopt.StringLong("subject", 's', "", "Set message subject.")
+	flagFastOff := getopt.BoolLong("fast-off", 0, "Disable XEP-0484: Fast Authentication Streamlining Tokens.")
 
 	// Parse command line flags.
 	getopt.Parse()
@@ -270,11 +267,13 @@ func main() {
 		fmt.Println(err)
 	}
 
-	fast, _ = getFastData(user, password)
-	// Reset FAST token and mechanism if expired.
-	if time.Now().After(fast.Expiry) {
-		fast.Token = ""
-		fast.Mechanism = ""
+	if !*flagFastOff {
+		fast, _ = getFastData(user, password)
+		// Reset FAST token and mechanism if expired.
+		if time.Now().After(fast.Expiry) {
+			fast.Token = ""
+			fast.Mechanism = ""
+		}
 	}
 
 	// Use ALPN
@@ -320,6 +319,7 @@ func main() {
 		SSDP:          !*flagSSDPOff,
 		UserAgentSW:   resource,
 		UserAgentID:   clientID,
+		Fast:          !*flagFastOff,
 		FastToken:     fast.Token,
 		FastMechanism: fast.Mechanism,
 	}
@@ -563,7 +563,7 @@ func main() {
 				default:
 					if err != nil {
 						cancel()
-						closeAndExit(client, errors.New("failed to read from stdin"))
+						closeAndExit(client, fmt.Errorf("failed to read from stdin"))
 					}
 				}
 			}
@@ -581,7 +581,7 @@ func main() {
 						continue
 					}
 					oxMessage, err := oxEncrypt(client, oxPrivKey,
-						recipient.Jid, recipient.OxKeyRing, message)
+						recipient.Jid, recipient.OxKeyRing, message, *flagSubject)
 					if err != nil {
 						fmt.Println("Ox: couldn't encrypt to",
 							recipient.Jid)
@@ -596,6 +596,7 @@ func main() {
 					_, err = client.Send(xmpp.Chat{
 						Remote: recipient.Jid,
 						Type:   msgType, Text: message,
+						Subject: *flagSubject,
 					})
 					if err != nil {
 						cancel()
@@ -692,6 +693,7 @@ func main() {
 				_, err = client.Send(xmpp.Chat{
 					Remote: recipient.Jid,
 					Type:   msgType, Ooburl: message, Text: message,
+					Subject: *flagSubject,
 				})
 				if err != nil {
 					fmt.Println("Couldn't send message to",
@@ -700,9 +702,15 @@ func main() {
 			// (Hopefully) temporary workaround due to go-xmpp choking on URL encoding.
 			// Once this is fixed in the lib the http-upload case above can be reused.
 			case *flagOOBFile != "":
-				_, err = client.SendOrg("<message to='" + recipient.Jid + "' type='" +
-					msgType + "'><body>" + message + "</body><x xmlns='jabber:x:oob'><url>" +
-					message + "</url></x></message>")
+				var msg string
+				if *flagSubject != "" {
+					msg = fmt.Sprintf("<message to='%s' type='%s'><subject>%s</subject><body>%s</body><x xmlns='jabber:x:oob'><url>%s</url></x></message>",
+						recipient.Jid, msgType, *flagSubject, message, message)
+				} else {
+					msg = fmt.Sprintf("<message to='%s' type='%s'><body>%s</body><x xmlns='jabber:x:oob'><url>%s</url></x></message>",
+						recipient.Jid, msgType, message, message)
+				}
+				_, err = client.SendOrg(msg)
 				if err != nil {
 					fmt.Println("Couldn't send message to",
 						recipient.Jid)
@@ -712,7 +720,7 @@ func main() {
 					continue
 				}
 				oxMessage, err := oxEncrypt(client, oxPrivKey,
-					recipient.Jid, recipient.OxKeyRing, message)
+					recipient.Jid, recipient.OxKeyRing, message, *flagSubject)
 				if err != nil {
 					fmt.Println("Ox: couldn't encrypt to", recipient.Jid)
 					continue
@@ -726,6 +734,7 @@ func main() {
 				_, err = client.Send(xmpp.Chat{
 					Remote: recipient.Jid,
 					Type:   msgType, Text: message,
+					Subject: *flagSubject,
 				})
 				if err != nil {
 					cancel()
